@@ -344,6 +344,7 @@ gdm_daemon_check_permissions (uid_t uid,
 {
         struct stat statbuf;
         const char *auth_path;
+        int         res;
 
         auth_path = LOGDIR;
 
@@ -353,7 +354,12 @@ gdm_daemon_check_permissions (uid_t uid,
         set_effective_user_group (0, 0);
 
         /* Now set things up for us as  */
-        chown (auth_path, 0, gid);
+        res = chown (auth_path, 0, gid);
+        if (res == -1) {
+                g_warning ("Unable to change owner for auth file: %s",
+                           g_strerror (errno));
+        }
+
         g_chmod (auth_path, (S_IRWXU|S_IRWXG|S_ISVTX));
 
         set_effective_user_group (uid, gid);
@@ -499,6 +505,20 @@ signal_cb (int      signo,
         return ret;
 }
 
+static gboolean
+is_debug_set (void)
+{
+        gboolean debug = FALSE;
+
+        /* enable debugging for unstable builds */
+        if (gdm_is_version_unstable ()) {
+                return TRUE;
+        }
+
+        gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
+        return debug;
+}
+
 int
 main (int    argc,
       char **argv)
@@ -512,12 +532,10 @@ main (int    argc,
         gboolean            res;
         gboolean            xdmcp_enabled;
         GdmSignalHandler   *signal_handler;
-        static gboolean     debug            = FALSE;
         static gboolean     do_timed_exit    = FALSE;
         static gboolean     print_version    = FALSE;
         static gboolean     fatal_warnings   = FALSE;
         static GOptionEntry entries []   = {
-                { "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable debugging code"), NULL },
                 { "fatal-warnings", 0, 0, G_OPTION_ARG_NONE, &fatal_warnings, N_("Make all warnings fatal"), NULL },
                 { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time - for debugging"), NULL },
                 { "version", 0, 0, G_OPTION_ARG_NONE, &print_version, N_("Print GDM version"), NULL },
@@ -548,6 +566,11 @@ main (int    argc,
                 goto out;
         }
 
+        if (print_version) {
+                g_print ("GDM %s\n", VERSION);
+                exit (1);
+        }
+
         if (fatal_warnings) {
                 GLogLevelFlags fatal_mask;
 
@@ -585,7 +608,7 @@ main (int    argc,
                 goto out;
         }
 
-        gdm_log_set_debug (debug);
+        gdm_log_set_debug (is_debug_set ());
 
         gdm_daemon_change_user (&gdm_uid, &gdm_gid);
         gdm_daemon_check_permissions (gdm_uid, gdm_gid);
@@ -596,7 +619,7 @@ main (int    argc,
         /* XDM compliant error message */
         if (getuid () != 0) {
                 /* make sure the pid file doesn't get wiped */
-                g_warning (_("Only root wants to run GDM"));
+                g_warning (_("Only the root user can run GDM"));
                 exit (-1);
         }
 
